__all__ = [
    'init', 'Clock', 'Color', 'GameClock', 'Font', 'Rect', 'Renderer', 'Surface', 'SysFont', 'Texture', 'Window',
    'sdlconstants', 'sdlkeys', 'sdl_ffi', 'image_ffi', 'ttf_ffi', 'sdl_lib', 'image_lib', 'ttf_lib',
    'color', 'display', 'event', 'font', 'gameclock', 'image', 'rect', 'renderer', 'surface', 'time', 'texture',
    'window',
]


#==============================================================================
#      _____ _____  _      ___
#     / ____|  __ \| |    |__ \
#    | (___ | |  | | |       ) |
#     \___ \| |  | | |      / /
#     ____) | |__| | |____ / /_
#    |_____/|_____/|______|____|
#
#==============================================================================


#-----------------------------------
# FFI, Dynamic libs, Constants, Keys
#-----------------------------------

from . import sdlffi
from . import sdllibs
from . import sdlconstants
from . import sdlkeys

sdl_ffi = sdlffi.sdl_ffi
image_ffi = sdlffi.image_ffi
ttf_ffi = sdlffi.ttf_ffi
mixer_ffi = sdlffi.mixer_ffi

sdl_lib = sdllibs.sdl_lib
image_lib = sdllibs.image_lib
ttf_lib = sdllibs.ttf_lib
mixer_lib = sdllibs.mixer_lib



#==============================================================================
#      _____  _____ _____  _      ___
#     / ____|/ ____|  __ \| |    |__ \
#    | |  __| (___ | |  | | |       ) |
#    | | |_ |\___ \| |  | | |      / /
#    | |__| |____) | |__| | |____ / /_
#     \_____|_____/|_____/|______|____|
#
#==============================================================================


#-----------------------------------
# Modules
#-----------------------------------

from . import color
from . import event
from . import image
from . import rect
from . import renderer
from . import surface
from . import font
from . import _time as time
from . import texture
from . import window
from . import display


#-----------------------------------
# Classes
#-----------------------------------

from color import Color
from ._time import Clock, GameClock
from font import Font, SysFont
from .rect import Rect
from .renderer import Renderer
from .surface import Surface
from .texture import Texture
from .window import Window


#-----------------------------------
# Startup / Shutdown
#-----------------------------------

import atexit


def init():
    sdl_lib.SDL_Init(sdlconstants.SDL_INIT_EVERYTHING)
    image_lib.IMG_Init(sdlconstants.IMG_INIT_EVERYTHING)
    ttf_lib.TTF_Init()

    atexit.register(_atexit)


def quit():
    sdl_lib.SDL_Quit()
    image_lib.IMG_Quit()
    ttf_lib.TTF_Quit()
    mixer_lib.Mix_CloseAudio()


def _atexit():
    # trying to encourage pypy to collect garbage
    del window.get_list()[:]
    display.Runtime.window = None
    display.Runtime.renderer = None
